@@ -21,6 +21,11 @@ class AuthSession;
 struct MessageGroupId;
 struct SendingAlbum;
 enum class SendMediaType;
+struct FileLoadTo;
+
+namespace InlineBots {
+class Result;
+} // namespace InlineBots
 
 namespace Storage {
 enum class SharedMediaType : signed char;
@@ -80,6 +85,8 @@ public:
 	//void setFeedChannels(
 	//	not_null<Data::Feed*> feed,
 	//	const std::vector<not_null<ChannelData*>> &channels);
+	void changeDialogUnreadMark(not_null<History*> history, bool unread);
+	//void changeDialogUnreadMark(not_null<Data::Feed*> feed, bool unread); // #feed
 
 	void requestFullPeer(PeerData *peer);
 	void requestPeer(PeerData *peer);
@@ -221,8 +228,7 @@ public:
 		Fn<void()> callbackNotModified = nullptr);
 
 	struct SendOptions {
-		SendOptions(not_null<History*> history) : history(history) {
-		}
+		SendOptions(not_null<History*> history);
 
 		not_null<History*> history;
 		MsgId replyTo = 0;
@@ -277,6 +283,21 @@ public:
 		bool silent);
 	void cancelLocalItem(not_null<HistoryItem*> item);
 
+	struct MessageToSend {
+		MessageToSend(not_null<History*> history);
+
+		not_null<History*> history;
+		TextWithTags textWithTags;
+		MsgId replyTo = 0;
+		WebPageId webPageId = 0;
+		bool clearDraft = true;
+	};
+	void sendMessage(MessageToSend &&message);
+	void sendInlineResult(
+		not_null<UserData*> bot,
+		not_null<InlineBots::Result*> data,
+		const SendOptions &options);
+
 	~ApiWrap();
 
 private:
@@ -439,6 +460,7 @@ private:
 		const MTPInputMedia &media,
 		bool silent,
 		uint64 randomId);
+	FileLoadTo fileLoadTaskOptions(const SendOptions &options) const;
 
 	void readFeeds();
 
