@@ -66,9 +66,6 @@ enum RoundCorners {
 namespace App {
 	MainWindow *wnd();
 	MainWidget *main();
-	bool passcoded();
-
-	void logOut();
 
 	QString formatPhone(QString phone);
 
@@ -135,7 +132,9 @@ namespace App {
 	inline ChannelData *channelLoaded(ChannelId channelId) {
 		return channel(channelId, PeerData::FullLoaded);
 	}
-	void enumerateUsers(base::lambda<void(UserData*)> action);
+	void enumerateUsers(Fn<void(not_null<UserData*>)> action);
+	void enumerateChatsChannels(
+		Fn<void(not_null<PeerData*>)> action);
 
 	UserData *self();
 	PeerData *peerByName(const QString &username);
@@ -221,16 +220,6 @@ namespace App {
 	QImage readImage(const QString &file, QByteArray *format = nullptr, bool opaque = true, bool *animated = nullptr, QByteArray *content = 0);
 	QPixmap pixmapFromImageInPlace(QImage &&image);
 
-	void regMuted(not_null<PeerData*> peer, TimeMs changeIn);
-	void unregMuted(not_null<PeerData*> peer);
-	void updateMuted();
-
-	void setProxySettings(QNetworkAccessManager &manager);
-#ifndef TDESKTOP_DISABLE_NETWORK_PROXY
-	QNetworkProxy getHttpProxySettings();
-#endif // !TDESKTOP_DISABLE_NETWORK_PROXY
-	void setProxySettings(QTcpSocket &socket);
-
 	void complexOverlayRect(Painter &p, QRect rect, ImageRoundRadius radius, RectParts corners);
 	void complexLocationRect(Painter &p, QRect rect, ImageRoundRadius radius, RectParts corners);
 
