@@ -38,4 +38,21 @@ QMap<QString, QString> url_parse_params(
 	return result;
 }
 
+bool is_ipv6(const QString &ip) {
+	//static const auto regexp = QRegularExpression("^[a-fA-F0-9:]+$");
+	//return regexp.match(ip).hasMatch();
+	return ip.indexOf('.') < 0 && ip.indexOf(':') >= 0;
+}
+
+QString url_append_query_or_hash(const QString &url, const QString &add) {
+	const auto query = url.lastIndexOf('?');
+	if (query < 0) {
+		return url + '?' + add;
+	}
+	const auto hash = url.lastIndexOf('#');
+	return url
+		+ (query >= 0 && query > url.lastIndexOf('#') ? '&' : '?')
+		+ add;
+}
+
 } // namespace qthelp
