@@ -31,18 +31,18 @@ void AboutBox::prepare() {
 
 	addButton(langFactory(lng_close), [this] { closeBox(); });
 
-	const auto linkHook = [](const ClickHandlerPtr &link, auto button) {
+	const auto linkFilter = [](const ClickHandlerPtr &link, auto button) {
 		if (const auto url = dynamic_cast<UrlClickHandler*>(link.get())) {
-			url->UrlClickHandler::onClick(button);
+			url->UrlClickHandler::onClick({ button });
 			return false;
 		}
 		return true;
 	};
 
 	_text3->setRichText(lng_about_text_3(lt_faq_open, qsl("[a href=\"%1\"]").arg(telegramFaqLink()), lt_faq_close, qsl("[/a]")));
-	_text1->setClickHandlerHook(linkHook);
-	_text2->setClickHandlerHook(linkHook);
-	_text3->setClickHandlerHook(linkHook);
+	_text1->setClickHandlerFilter(linkFilter);
+	_text2->setClickHandlerFilter(linkFilter);
+	_text3->setClickHandlerFilter(linkFilter);
 
 	_version->setClickedCallback([this] { showVersionHistory(); });
 
