@@ -32,7 +32,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 namespace {
 
-constexpr auto kMaxGroupChannelTitle = 255;
+constexpr auto kMaxGroupChannelTitle = 255; // See also edit_peer_info_box.
 constexpr auto kMaxChannelDescription = 255; // See also edit_peer_info_box.
 constexpr auto kMaxBioLength = 70;
 constexpr auto kMinUsernameLength = 5;
@@ -57,7 +57,7 @@ QString PeerFloodErrorText(PeerFloodType type) {
 
 class RevokePublicLinkBox::Inner : public TWidget, private MTP::Sender {
 public:
-	Inner(QWidget *parent, base::lambda<void()> revokeCallback);
+	Inner(QWidget *parent, Fn<void()> revokeCallback);
 
 protected:
 	void mouseMoveEvent(QMouseEvent *e) override;
@@ -85,7 +85,7 @@ private:
 	int _rowHeight = 0;
 	int _revokeWidth = 0;
 
-	base::lambda<void()> _revokeCallback;
+	Fn<void()> _revokeCallback;
 	mtpRequestId _revokeRequestId = 0;
 	QPointer<ConfirmBox> _weakRevokeConfirmBox;
 
@@ -122,9 +122,9 @@ void AddContactBox::prepare() {
 	}
 	updateButtons();
 
-	connect(_first, SIGNAL(submitted(bool)), this, SLOT(onSubmit()));
-	connect(_last, SIGNAL(submitted(bool)), this, SLOT(onSubmit()));
-	connect(_phone, SIGNAL(submitted(bool)), this, SLOT(onSubmit()));
+	connect(_first, &Ui::InputField::submitted, [=] { submit(); });
+	connect(_last, &Ui::InputField::submitted, [=] { submit(); });
+	connect(_phone, &Ui::PhoneInput::submitted, [=] { submit(); });
 
 	setDimensions(st::boxWideWidth, st::contactPadding.top() + _first->height() + st::contactSkip + _last->height() + st::contactPhoneSkip + _phone->height() + st::contactPadding.bottom() + st::boxPadding.bottom());
 }
@@ -170,21 +170,21 @@ void AddContactBox::resizeEvent(QResizeEvent *e) {
 	}
 }
 
-void AddContactBox::onSubmit() {
+void AddContactBox::submit() {
 	if (_first->hasFocus()) {
 		_last->setFocus();
 	} else if (_last->hasFocus()) {
 		if (_phone->isEnabled()) {
 			_phone->setFocus();
 		} else {
-			onSave();
+			save();
 		}
 	} else if (_phone->hasFocus()) {
-		onSave();
+		save();
 	}
 }
 
-void AddContactBox::onSave() {
+void AddContactBox::save() {
 	if (_addRequest) return;
 
 	auto firstName = TextUtilities::PrepareForSending(_first->getLastText());
@@ -274,7 +274,7 @@ void AddContactBox::onSaveUserDone(const MTPcontacts_ImportedContacts &res) {
 	closeBox();
 }
 
-void AddContactBox::onRetry() {
+void AddContactBox::retry() {
 	_addRequest = 0;
 	_contactId = 0;
 	showChildren();
@@ -291,9 +291,9 @@ void AddContactBox::onRetry() {
 void AddContactBox::updateButtons() {
 	clearButtons();
 	if (_retrying) {
-		addButton(langFactory(lng_try_other_contact), [this] { onRetry(); });
+		addButton(langFactory(lng_try_other_contact), [this] { retry(); });
 	} else {
-		addButton(langFactory(_user ? lng_settings_save : lng_add_contact), [this] { onSave(); });
+		addButton(langFactory(_user ? lng_settings_save : lng_add_contact), [this] { save(); });
 		addButton(langFactory(lng_cancel), [this] { closeBox(); });
 	}
 }
@@ -320,20 +320,29 @@ void GroupInfoBox::prepare() {
 			? lng_dlg_new_channel_name
 			: lng_dlg_new_group_name));
 	_title->setMaxLength(kMaxGroupChannelTitle);
+	_title->setInstantReplaces(Ui::InstantReplaces::Default());
+	_title ->setInstantReplacesEnabled(Global::ReplaceEmojiValue());
 
 	if (_creating == CreatingGroupChannel) {
-		_description.create(this, st::newGroupDescription, langFactory(lng_create_group_description));
+		_description.create(
+			this,
+			st::newGroupDescription,
+			Ui::InputField::Mode::MultiLine,
+			langFactory(lng_create_group_description));
 		_description->show();
 		_description->setMaxLength(kMaxChannelDescription);
+		_description->setInstantReplaces(Ui::InstantReplaces::Default());
+		_description->setInstantReplacesEnabled(
+			Global::ReplaceEmojiValue());
 
-		connect(_description, SIGNAL(resized()), this, SLOT(onDescriptionResized()));
-		connect(_description, SIGNAL(submitted(bool)), this, SLOT(onNext()));
-		connect(_description, SIGNAL(cancelled()), this, SLOT(onClose()));
+		connect(_description, &Ui::InputField::resized, [=] { descriptionResized(); });
+		connect(_description, &Ui::InputField::submitted, [=] { submit(); });
+		connect(_description, &Ui::InputField::cancelled, [=] { closeBox(); });
 	}
 
-	connect(_title, SIGNAL(submitted(bool)), this, SLOT(onNameSubmit()));
+	connect(_title, &Ui::InputField::submitted, [=] { submitName(); });
 
-	addButton(langFactory(_creating == CreatingGroupChannel ? lng_create_group_create : lng_create_group_next), [this] { onNext(); });
+	addButton(langFactory(_creating == CreatingGroupChannel ? lng_create_group_create : lng_create_group_next), [this] { submit(); });
 	addButton(langFactory(_fromTypeChoose ? lng_create_group_back : lng_cancel), [this] { closeBox(); });
 
 	updateMaxHeight();
@@ -364,14 +373,14 @@ void GroupInfoBox::resizeEvent(QResizeEvent *e) {
 	}
 }
 
-void GroupInfoBox::onNameSubmit() {
+void GroupInfoBox::submitName() {
 	if (_title->getLastText().trimmed().isEmpty()) {
 		_title->setFocus();
 		_title->showError();
 	} else if (_description) {
 		_description->setFocus();
 	} else {
-		onNext();
+		submit();
 	}
 }
 
@@ -451,11 +460,15 @@ void GroupInfoBox::createGroup(not_null<PeerListBox*> selectUsersBox, const QStr
 	}).send();
 }
 
-void GroupInfoBox::onNext() {
+void GroupInfoBox::submit() {
 	if (_creationRequestId) return;
 
 	auto title = TextUtilities::PrepareForSending(_title->getLastText());
-	auto description = _description ? TextUtilities::PrepareForSending(_description->getLastText(), TextUtilities::PrepareTextOption::CheckLinks) : QString();
+	auto description = _description
+		? TextUtilities::PrepareForSending(
+			_description->getLastText(),
+			TextUtilities::PrepareTextOption::CheckLinks)
+		: QString();
 	if (title.isEmpty()) {
 		_title->setFocus();
 		_title->showError();
@@ -551,7 +564,7 @@ void GroupInfoBox::createChannel(const QString &title, const QString &descriptio
 	}).send();
 }
 
-void GroupInfoBox::onDescriptionResized() {
+void GroupInfoBox::descriptionResized() {
 	updateMaxHeight();
 	update();
 }
@@ -579,7 +592,7 @@ SetupChannelBox::SetupChannelBox(QWidget*, ChannelData *channel, bool existing)
 , _aboutPublicWidth(st::boxWideWidth - st::boxPadding.left() - st::boxButtonPadding.right() - st::newGroupPadding.left() - st::defaultRadio.diameter - st::defaultBoxCheckbox.textPosition.x())
 , _aboutPublic(st::defaultTextStyle, lang(channel->isMegagroup() ? lng_create_public_group_about : lng_create_public_channel_about), _defaultOptions, _aboutPublicWidth)
 , _aboutPrivate(st::defaultTextStyle, lang(channel->isMegagroup() ? lng_create_private_group_about : lng_create_private_channel_about), _defaultOptions, _aboutPublicWidth)
-, _link(this, st::setupChannelLink, base::lambda<QString()>(), channel->username, true) {
+, _link(this, st::setupChannelLink, Fn<QString()>(), channel->username, true) {
 }
 
 void SetupChannelBox::prepare() {
@@ -589,14 +602,14 @@ void SetupChannelBox::prepare() {
 
 	_checkRequestId = MTP::send(MTPchannels_CheckUsername(_channel->inputChannel, MTP_string("preston")), RPCDoneHandlerPtr(), rpcFail(&SetupChannelBox::onFirstCheckFail));
 
-	addButton(langFactory(lng_settings_save), [this] { onSave(); });
-	addButton(langFactory(_existing ? lng_cancel : lng_create_group_skip), [this] { closeBox(); });
+	addButton(langFactory(lng_settings_save), [=] { save(); });
+	addButton(langFactory(_existing ? lng_cancel : lng_create_group_skip), [=] { closeBox(); });
 
-	connect(_link, SIGNAL(changed()), this, SLOT(onChange()));
+	connect(_link, &Ui::MaskedInputField::changed, [=] { handleChange(); });
 	_link->setVisible(_privacyGroup->value() == Privacy::Public);
 
 	_checkTimer.setSingleShot(true);
-	connect(&_checkTimer, SIGNAL(timeout()), this, SLOT(onCheck()));
+	connect(&_checkTimer, &QTimer::timeout, [=] { check(); });
 
 	_privacyGroup->setChangedCallback([this](Privacy value) { privacyChanged(value); });
 	subscribe(Notify::PeerUpdated(), Notify::PeerUpdatedHandler(Notify::PeerUpdate::Flag::InviteLinkChanged, [this](const Notify::PeerUpdate &update) {
@@ -604,11 +617,13 @@ void SetupChannelBox::prepare() {
 			rtlupdate(_invitationLink);
 		}
 	}));
-	subscribe(boxClosing, [this] {
+
+	boxClosing() | rpl::start_with_next([=] {
 		if (!_existing) {
 			AddParticipantsBoxController::Start(_channel);
 		}
-	});
+	}, lifetime());
+
 	updateMaxHeight();
 }
 
@@ -635,7 +650,7 @@ void SetupChannelBox::keyPressEvent(QKeyEvent *e) {
 				_link->setFocus();
 				_link->showError();
 			} else {
-				onSave();
+				save();
 			}
 		}
 	} else {
@@ -724,7 +739,7 @@ void SetupChannelBox::updateSelected(const QPoint &cursorGlobalPosition) {
 	}
 }
 
-void SetupChannelBox::onSave() {
+void SetupChannelBox::save() {
 	if (_privacyGroup->value() == Privacy::Private) {
 		if (_existing) {
 			_sentUsername = QString();
@@ -747,7 +762,7 @@ void SetupChannelBox::onSave() {
 	_saveRequestId = MTP::send(MTPchannels_UpdateUsername(_channel->inputChannel, MTP_string(_sentUsername)), rpcDone(&SetupChannelBox::onUpdateDone), rpcFail(&SetupChannelBox::onUpdateFail));
 }
 
-void SetupChannelBox::onChange() {
+void SetupChannelBox::handleChange() {
 	QString name = _link->text().trimmed();
 	if (name.isEmpty()) {
 		if (!_errorText.isEmpty() || !_goodText.isEmpty()) {
@@ -784,7 +799,7 @@ void SetupChannelBox::onChange() {
 	}
 }
 
-void SetupChannelBox::onCheck() {
+void SetupChannelBox::check() {
 	if (_checkRequestId) {
 		MTP::cancel(_checkRequestId);
 	}
@@ -804,10 +819,10 @@ void SetupChannelBox::privacyChanged(Privacy value) {
 	if (value == Privacy::Public) {
 		if (_tooMuchUsernames) {
 			_privacyGroup->setValue(Privacy::Private);
-			Ui::show(Box<RevokePublicLinkBox>(base::lambda_guarded(this, [this] {
+			Ui::show(Box<RevokePublicLinkBox>(crl::guard(this, [this] {
 				_tooMuchUsernames = false;
 				_privacyGroup->setValue(Privacy::Public);
-				onCheck();
+				check();
 			})), LayerOption::KeepOther);
 			return;
 		}
@@ -943,16 +958,16 @@ void EditNameBox::prepare() {
 	newHeight += st::boxPadding.bottom() + st::contactPadding.bottom();
 	setDimensions(st::boxWideWidth, newHeight);
 
-	addButton(langFactory(lng_settings_save), [this] { save(); });
-	addButton(langFactory(lng_cancel), [this] { closeBox(); });
+	addButton(langFactory(lng_settings_save), [=] { save(); });
+	addButton(langFactory(lng_cancel), [=] { closeBox(); });
 	if (_invertOrder) {
 		setTabOrder(_last, _first);
 	}
 	_first->setMaxLength(kMaxGroupChannelTitle);
 	_last->setMaxLength(kMaxGroupChannelTitle);
 
-	connect(_first, &Ui::InputField::submitted, this, [this] { submit(); });
-	connect(_last, &Ui::InputField::submitted, this, [this] { submit(); });
+	connect(_first, &Ui::InputField::submitted, [=] { submit(); });
+	connect(_last, &Ui::InputField::submitted, [=] { submit(); });
 }
 
 void EditNameBox::setInnerFocus() {
@@ -1052,7 +1067,12 @@ bool EditNameBox::saveSelfFail(const RPCError &error) {
 EditBioBox::EditBioBox(QWidget*, not_null<UserData*> self) : BoxContent()
 , _dynamicFieldStyle(CreateBioFieldStyle())
 , _self(self)
-, _bio(this, _dynamicFieldStyle, langFactory(lng_bio_placeholder), _self->about())
+, _bio(
+	this,
+	_dynamicFieldStyle,
+	Ui::InputField::Mode::MultiLine,
+	langFactory(lng_bio_placeholder),
+	_self->about())
 , _countdown(this, QString(), Ui::FlatLabel::InitType::Simple, st::editBioCountdownLabel)
 , _about(this, lang(lng_bio_about), Ui::FlatLabel::InitType::Simple, st::aboutRevokePublicLabel) {
 }
@@ -1063,13 +1083,15 @@ void EditBioBox::prepare() {
 	addButton(langFactory(lng_settings_save), [this] { save(); });
 	addButton(langFactory(lng_cancel), [this] { closeBox(); });
 	_bio->setMaxLength(kMaxBioLength);
-	_bio->setCtrlEnterSubmit(Ui::CtrlEnterSubmit::Both);
+	_bio->setSubmitSettings(Ui::InputField::SubmitSettings::Both);
 	auto cursor = _bio->textCursor();
 	cursor.setPosition(_bio->getLastText().size());
 	_bio->setTextCursor(cursor);
-	connect(_bio, &Ui::InputArea::submitted, this, [this](bool ctrlShiftEnter) { save(); });
-	connect(_bio, &Ui::InputArea::resized, this, [this] { updateMaxHeight(); });
-	connect(_bio, &Ui::InputArea::changed, this, [this] { handleBioUpdated(); });
+	connect(_bio, &Ui::InputField::submitted, [=] { save(); });
+	connect(_bio, &Ui::InputField::resized, [=] { updateMaxHeight(); });
+	connect(_bio, &Ui::InputField::changed, [=] { handleBioUpdated(); });
+	_bio->setInstantReplaces(Ui::InstantReplaces::Default());
+	_bio->setInstantReplacesEnabled(Global::ReplaceEmojiValue());
 	handleBioUpdated();
 	updateMaxHeight();
 }
@@ -1122,7 +1144,12 @@ void EditBioBox::save() {
 EditChannelBox::EditChannelBox(QWidget*, not_null<ChannelData*> channel)
 : _channel(channel)
 , _title(this, st::defaultInputField, langFactory(_channel->isMegagroup() ? lng_dlg_new_group_name : lng_dlg_new_channel_name), _channel->name)
-, _description(this, st::newGroupDescription, langFactory(lng_create_group_description), _channel->about())
+, _description(
+	this,
+	st::newGroupDescription,
+	Ui::InputField::Mode::MultiLine,
+	langFactory(lng_create_group_description),
+	_channel->about())
 , _sign(this, lang(lng_edit_sign_messages), channel->addsSignature(), st::defaultBoxCheckbox)
 , _inviteGroup(std::make_shared<Ui::RadioenumGroup<Invites>>(channel->anyoneCanAddMembers() ? Invites::Everybody : Invites::OnlyAdmins))
 , _inviteEverybody(this, _inviteGroup, Invites::Everybody, lang(lng_edit_group_invites_everybody))
@@ -1133,7 +1160,7 @@ EditChannelBox::EditChannelBox(QWidget*, not_null<ChannelData*> channel)
 void EditChannelBox::prepare() {
 	setTitle(langFactory(_channel->isMegagroup() ? lng_edit_group : lng_edit_channel_title));
 
-	addButton(langFactory(lng_settings_save), [this] { onSave(); });
+	addButton(langFactory(lng_settings_save), [this] { save(); });
 	addButton(langFactory(lng_cancel), [this] { closeBox(); });
 
 	subscribe(Notify::PeerUpdated(), Notify::PeerUpdatedHandler(Notify::PeerUpdate::Flag::NameChanged, [this](const Notify::PeerUpdate &update) {
@@ -1145,13 +1172,17 @@ void EditChannelBox::prepare() {
 	setMouseTracking(true);
 
 	_title->setMaxLength(kMaxGroupChannelTitle);
+	_title->setInstantReplaces(Ui::InstantReplaces::Default());
+	_title->setInstantReplacesEnabled(Global::ReplaceEmojiValue());
 	_description->setMaxLength(kMaxChannelDescription);
+	_description->setInstantReplaces(Ui::InstantReplaces::Default());
+	_description->setInstantReplacesEnabled(Global::ReplaceEmojiValue());
 
-	connect(_description, SIGNAL(resized()), this, SLOT(onDescriptionResized()));
-	connect(_description, SIGNAL(submitted(bool)), this, SLOT(onSave()));
-	connect(_description, SIGNAL(cancelled()), this, SLOT(onClose()));
+	connect(_description, &Ui::InputField::resized, [=] { descriptionResized(); });
+	connect(_description, &Ui::InputField::submitted, [=] { save(); });
+	connect(_description, &Ui::InputField::cancelled, [=] { closeBox(); });
 
-	connect(_publicLink, SIGNAL(clicked()), this, SLOT(onPublicLink()));
+	_publicLink->addClickHandler([=] { setupPublicLink(); });
 	_publicLink->setVisible(_channel->canEditUsername());
 	_sign->setVisible(canEditSignatures());
 	_inviteEverybody->setVisible(canEditInvites());
@@ -1167,7 +1198,7 @@ void EditChannelBox::setInnerFocus() {
 void EditChannelBox::keyPressEvent(QKeyEvent *e) {
 	if (e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return) {
 		if (_title->hasFocus()) {
-			onSave();
+			save();
 		}
 	} else {
 		BoxContent::keyPressEvent(e);
@@ -1179,7 +1210,7 @@ void EditChannelBox::handleChannelNameChange() {
 	_sign->setChecked(_channel->addsSignature());
 }
 
-void EditChannelBox::onDescriptionResized() {
+void EditChannelBox::descriptionResized() {
 	updateMaxHeight();
 	update();
 }
@@ -1243,7 +1274,7 @@ void EditChannelBox::paintEvent(QPaintEvent *e) {
 	}
 }
 
-void EditChannelBox::onSave() {
+void EditChannelBox::save() {
 	if (_saveTitleRequestId || _saveDescriptionRequestId || _saveSignRequestId || _saveInvitesRequestId) return;
 
 	auto title = TextUtilities::PrepareForSending(_title->getLastText());
@@ -1262,7 +1293,7 @@ void EditChannelBox::onSave() {
 	}
 }
 
-void EditChannelBox::onPublicLink() {
+void EditChannelBox::setupPublicLink() {
 	Ui::show(
 		Box<SetupChannelBox>(_channel, true),
 		LayerOption::KeepOther);
@@ -1358,7 +1389,7 @@ void EditChannelBox::onSaveInvitesDone(const MTPUpdates &result) {
 	closeBox();
 }
 
-RevokePublicLinkBox::Inner::Inner(QWidget *parent, base::lambda<void()> revokeCallback) : TWidget(parent)
+RevokePublicLinkBox::Inner::Inner(QWidget *parent, Fn<void()> revokeCallback) : TWidget(parent)
 , _rowHeight(st::contactsPadding.top() + st::contactsPhotoSize + st::contactsPadding.bottom())
 , _revokeWidth(st::normalFont->width(lang(lng_channels_too_much_public_revoke)))
 , _revokeCallback(std::move(revokeCallback)) {
@@ -1394,7 +1425,7 @@ RevokePublicLinkBox::Inner::Inner(QWidget *parent, base::lambda<void()> revokeCa
 	}).send();
 }
 
-RevokePublicLinkBox::RevokePublicLinkBox(QWidget*, base::lambda<void()> revokeCallback)
+RevokePublicLinkBox::RevokePublicLinkBox(QWidget*, Fn<void()> revokeCallback)
 : _aboutRevoke(this, lang(lng_channels_too_much_public_about), Ui::FlatLabel::InitType::Simple, st::aboutRevokePublicLabel)
 , _revokeCallback(std::move(revokeCallback)) {
 }
@@ -1453,7 +1484,7 @@ void RevokePublicLinkBox::Inner::mouseReleaseEvent(QMouseEvent *e) {
 		auto text_method = pressed->isMegagroup() ? lng_channels_too_much_public_revoke_confirm_group : lng_channels_too_much_public_revoke_confirm_channel;
 		auto text = text_method(lt_link, Messenger::Instance().createInternalLink(pressed->userName()), lt_group, pressed->name);
 		auto confirmText = lang(lng_channels_too_much_public_revoke);
-		_weakRevokeConfirmBox = Ui::show(Box<ConfirmBox>(text, confirmText, base::lambda_guarded(this, [this, pressed]() {
+		_weakRevokeConfirmBox = Ui::show(Box<ConfirmBox>(text, confirmText, crl::guard(this, [this, pressed]() {
 			if (_revokeRequestId) return;
 			_revokeRequestId = request(MTPchannels_UpdateUsername(pressed->asChannel()->inputChannel, MTP_string(""))).done([this](const MTPBool &result) {
 				if (_weakRevokeConfirmBox) {
