@@ -16,6 +16,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_photo.h"
 #include "data/data_document.h"
 #include "lang/lang_keys.h"
+#include "chat_helpers/message_field.h"
 #include "window/window_controller.h"
 #include "mainwidget.h"
 #include "layout.h"
@@ -24,8 +25,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 EditCaptionBox::EditCaptionBox(
 	QWidget*,
+	not_null<Window::Controller*> controller,
 	not_null<HistoryItem*> item)
-: _msgId(item->fullId()) {
+: _controller(controller)
+, _msgId(item->fullId()) {
 	Expects(item->media() != nullptr);
 	Expects(item->media()->allowsEditCaption());
 
@@ -50,7 +53,11 @@ EditCaptionBox::EditCaptionBox(
 		}
 		doc = document;
 	}
-	auto caption = item->originalText().text;
+	const auto original = item->originalText();
+	const auto editData = TextWithTags {
+		original.text,
+		ConvertEntitiesToTextTags(original.entities)
+	};
 
 	if (!_animated && (dimensions.isEmpty() || doc || image->isNull())) {
 		if (image->isNull()) {
@@ -130,9 +137,19 @@ EditCaptionBox::EditCaptionBox(
 	}
 	Assert(_animated || _photo || _doc);
 
-	_field.create(this, st::confirmCaptionArea, langFactory(lng_photo_caption), caption);
+	_field.create(
+		this,
+		st::confirmCaptionArea,
+		Ui::InputField::Mode::MultiLine,
+		langFactory(lng_photo_caption),
+		editData);
 	_field->setMaxLength(MaxPhotoCaption);
-	_field->setCtrlEnterSubmit(Ui::CtrlEnterSubmit::Both);
+	_field->setSubmitSettings(Ui::InputField::SubmitSettings::Both);
+	_field->setInstantReplaces(Ui::InstantReplaces::Default());
+	_field->setInstantReplacesEnabled(Global::ReplaceEmojiValue());
+	_field->setMarkdownReplacesEnabled(rpl::single(true));
+	_field->setEditLinkCallback(
+		DefaultEditLinkCallback(_controller, _field));
 }
 
 void EditCaptionBox::prepareGifPreview(DocumentData *document) {
@@ -177,13 +194,9 @@ void EditCaptionBox::prepare() {
 	addButton(langFactory(lng_cancel), [this] { closeBox(); });
 
 	updateBoxSize();
-	connect(_field, &Ui::InputArea::submitted, this, [this] { save(); });
-	connect(_field, &Ui::InputArea::cancelled, this, [this] {
-		closeBox();
-	});
-	connect(_field, &Ui::InputArea::resized, this, [this] {
-		captionResized();
-	});
+	connect(_field, &Ui::InputField::submitted, [=] { save(); });
+	connect(_field, &Ui::InputField::cancelled, [=] { closeBox(); });
+	connect(_field, &Ui::InputField::resized, [=] { captionResized(); });
 
 	auto cursor = _field->textCursor();
 	cursor.movePosition(QTextCursor::End);
@@ -228,7 +241,7 @@ void EditCaptionBox::paintEvent(QPaintEvent *e) {
 		}
 		if (_gifPreview && _gifPreview->started()) {
 			auto s = QSize(_thumbw, _thumbh);
-			auto paused = controller()->isGifPausedAtLeastFor(Window::GifPauseReason::Layer);
+			auto paused = _controller->isGifPausedAtLeastFor(Window::GifPauseReason::Layer);
 			auto frame = _gifPreview->current(s.width(), s.height(), s.width(), s.height(), ImageRoundRadius::None, RectPart::None, paused ? 0 : getms());
 			p.drawPixmap(_thumbx, st::boxPhotoPadding.top(), frame);
 		} else {
@@ -332,17 +345,30 @@ void EditCaptionBox::save() {
 	if (_previewCancelled) {
 		flags |= MTPmessages_EditMessage::Flag::f_no_webpage;
 	}
-	MTPVector<MTPMessageEntity> sentEntities;
+	const auto textWithTags = _field->getTextWithAppliedMarkdown();
+	auto sending = TextWithEntities{
+		textWithTags.text,
+		ConvertTextTagsToEntities(textWithTags.tags)
+	};
+	const auto prepareFlags = Ui::ItemTextOptions(
+		item->history(),
+		App::self()).flags;
+	TextUtilities::PrepareForSending(sending, prepareFlags);
+	TextUtilities::Trim(sending);
+
+	const auto sentEntities = TextUtilities::EntitiesToMTP(
+		sending.entities,
+		TextUtilities::ConvertOption::SkipLocal);
 	if (!sentEntities.v.isEmpty()) {
 		flags |= MTPmessages_EditMessage::Flag::f_entities;
 	}
-	auto text = TextUtilities::PrepareForSending(_field->getLastText(), TextUtilities::PrepareTextOption::CheckLinks);
 	_saveRequestId = MTP::send(
 		MTPmessages_EditMessage(
 			MTP_flags(flags),
 			item->history()->peer->input,
 			MTP_int(item->id),
-			MTP_string(text),
+			MTP_string(sending.text),
+			MTPInputMedia(),
 			MTPnullMarkup,
 			sentEntities,
 			MTP_inputGeoPointEmpty()),
