@@ -14,21 +14,13 @@ template <typename Enum>
 class RadioenumGroup;
 template <typename Enum>
 class Radioenum;
-class InputArea;
+class InputField;
 } // namespace Ui
 
 class ReportBox : public BoxContent, public RPCSender {
-	Q_OBJECT
-
 public:
-	ReportBox(QWidget*, PeerData *peer);
-
-private slots:
-	void onReport();
-	void onReasonResized();
-	void onClose() {
-		closeBox();
-	}
+	ReportBox(QWidget*, not_null<PeerData*> peer);
+	ReportBox(QWidget*, not_null<PeerData*> peer, MessageIdsList ids);
 
 protected:
 	void prepare() override;
@@ -44,19 +36,22 @@ private:
 		Other,
 	};
 	void reasonChanged(Reason reason);
+	void reasonResized();
 	void updateMaxHeight();
+	void report();
 
 	void reportDone(const MTPBool &result);
 	bool reportFail(const RPCError &error);
 
-	PeerData *_peer;
+	not_null<PeerData*> _peer;
+	base::optional<MessageIdsList> _ids;
 
 	std::shared_ptr<Ui::RadioenumGroup<Reason>> _reasonGroup;
-	object_ptr<Ui::Radioenum<Reason>> _reasonSpam;
-	object_ptr<Ui::Radioenum<Reason>> _reasonViolence;
-	object_ptr<Ui::Radioenum<Reason>> _reasonPornography;
-	object_ptr<Ui::Radioenum<Reason>> _reasonOther;
-	object_ptr<Ui::InputArea> _reasonOtherText = { nullptr };
+	object_ptr<Ui::Radioenum<Reason>> _reasonSpam = { nullptr };
+	object_ptr<Ui::Radioenum<Reason>> _reasonViolence = { nullptr };
+	object_ptr<Ui::Radioenum<Reason>> _reasonPornography = { nullptr };
+	object_ptr<Ui::Radioenum<Reason>> _reasonOther = { nullptr };
+	object_ptr<Ui::InputField> _reasonOtherText = { nullptr };
 
 	mtpRequestId _requestId = 0;
 
