@@ -14,11 +14,15 @@ namespace Ui {
 namespace Emoji {
 
 inline utf16string QStringToUTF16(const QString &string) {
-	return utf16string(reinterpret_cast<const utf16char*>(string.constData()), string.size());
+	return utf16string(
+		reinterpret_cast<const utf16char*>(string.constData()),
+		string.size());
 }
 
 inline QString QStringFromUTF16(utf16string string) {
-	return QString::fromRawData(reinterpret_cast<const QChar*>(string.data()), string.size());
+	return QString::fromRawData(
+		reinterpret_cast<const QChar*>(string.data()),
+		string.size());
 }
 
 constexpr auto kSuggestionMaxLength = internal::kReplacementMaxLength;
