@@ -103,7 +103,7 @@ QString UrlClickHandler::url() const {
 	return result;
 }
 
-void UrlClickHandler::doOpen(QString url) {
+void UrlClickHandler::Open(QString url, QVariant context) {
 	Ui::Tooltip::Hide();
 
 	if (isEmail(url)) {
@@ -114,7 +114,7 @@ void UrlClickHandler::doOpen(QString url) {
 	url = tryConvertUrlToLocal(url);
 
 	if (url.startsWith(qstr("tg://"), Qt::CaseInsensitive)) {
-		Messenger::Instance().openLocalUrl(url);
+		Messenger::Instance().openLocalUrl(url, context);
 	} else {
 		QDesktopServices::openUrl(url);
 	}
@@ -140,38 +140,52 @@ TextWithEntities UrlClickHandler::getExpandedLinkTextWithEntities(ExpandLinksMod
 	return result;
 }
 
-void HiddenUrlClickHandler::doOpen(QString url) {
+void HiddenUrlClickHandler::Open(QString url, QVariant context) {
 	auto urlText = tryConvertUrlToLocal(url);
 
 	if (urlText.startsWith(qstr("tg://"), Qt::CaseInsensitive)) {
-		Messenger::Instance().openLocalUrl(urlText);
+		Messenger::Instance().openLocalUrl(urlText, context);
 	} else {
+		const auto open = [=] {
+			UrlClickHandler::Open(urlText, context);
+		};
 		auto parsedUrl = QUrl::fromUserInput(urlText);
 		if (UrlRequiresConfirmation(urlText)) {
-			auto displayUrl = parsedUrl.isValid() ? parsedUrl.toDisplayString() : urlText;
-			Ui::show(Box<ConfirmBox>(lang(lng_open_this_link) + qsl("\n\n") + displayUrl, lang(lng_open_link), [urlText] {
-				Ui::hideLayer();
-				UrlClickHandler::doOpen(urlText);
-			}), LayerOption::KeepOther);
+			auto displayUrl = parsedUrl.isValid()
+				? parsedUrl.toDisplayString()
+				: urlText;
+			Ui::show(
+				Box<ConfirmBox>(
+					lang(lng_open_this_link) + qsl("\n\n") + displayUrl,
+					lang(lng_open_link),
+					[=] { Ui::hideLayer(); open(); }),
+				LayerOption::KeepOther);
 		} else {
-			UrlClickHandler::doOpen(urlText);
+			open();
 		}
 	}
 }
 
-void BotGameUrlClickHandler::onClick(Qt::MouseButton button) const {
+void BotGameUrlClickHandler::onClick(ClickContext context) const {
 	auto urlText = tryConvertUrlToLocal(url());
 
+	const auto open = [=] {
+		UrlClickHandler::Open(urlText, context.other);
+	};
 	if (urlText.startsWith(qstr("tg://"), Qt::CaseInsensitive)) {
-		Messenger::Instance().openLocalUrl(urlText);
+		Messenger::Instance().openLocalUrl(urlText, context.other);
 	} else if (!_bot || _bot->isVerified() || Local::isBotTrusted(_bot)) {
-		doOpen(urlText);
+		open();
 	} else {
-		Ui::show(Box<ConfirmBox>(lng_allow_bot_pass(lt_bot_name, _bot->name), lang(lng_allow_bot), [bot = _bot, urlText] {
+		const auto callback = [=, bot = _bot] {
 			Ui::hideLayer();
 			Local::makeBotTrusted(bot);
-			UrlClickHandler::doOpen(urlText);
-		}));
+			open();
+		};
+		Ui::show(Box<ConfirmBox>(
+			lng_allow_bot_pass(lt_bot_name, _bot->name),
+			lang(lng_allow_bot),
+			callback));
 	}
 }
 
@@ -203,7 +217,8 @@ QString MentionClickHandler::copyToClipboardContextItemText() const {
 	return lang(lng_context_copy_mention);
 }
 
-void MentionClickHandler::onClick(Qt::MouseButton button) const {
+void MentionClickHandler::onClick(ClickContext context) const {
+	const auto button = context.button;
 	if (button == Qt::LeftButton || button == Qt::MiddleButton) {
 		App::openPeerByName(_tag.mid(1), ShowAtProfileMsgId);
 	}
@@ -213,7 +228,8 @@ TextWithEntities MentionClickHandler::getExpandedLinkTextWithEntities(ExpandLink
 	return simpleTextWithEntity({ EntityInTextMention, entityOffset, textPart.size() });
 }
 
-void MentionNameClickHandler::onClick(Qt::MouseButton button) const {
+void MentionNameClickHandler::onClick(ClickContext context) const {
+	const auto button = context.button;
 	if (button == Qt::LeftButton || button == Qt::MiddleButton) {
 		if (auto user = App::userLoaded(_userId)) {
 			Ui::showPeerProfile(user);
@@ -240,7 +256,8 @@ QString HashtagClickHandler::copyToClipboardContextItemText() const {
 	return lang(lng_context_copy_hashtag);
 }
 
-void HashtagClickHandler::onClick(Qt::MouseButton button) const {
+void HashtagClickHandler::onClick(ClickContext context) const {
+	const auto button = context.button;
 	if (button == Qt::LeftButton || button == Qt::MiddleButton) {
 		App::searchByHashtag(_tag, Ui::getPeerForMouseAction());
 	}
@@ -254,7 +271,8 @@ QString CashtagClickHandler::copyToClipboardContextItemText() const {
 	return lang(lng_context_copy_hashtag);
 }
 
-void CashtagClickHandler::onClick(Qt::MouseButton button) const {
+void CashtagClickHandler::onClick(ClickContext context) const {
+	const auto button = context.button;
 	if (button == Qt::LeftButton || button == Qt::MiddleButton) {
 		App::searchByHashtag(_tag, Ui::getPeerForMouseAction());
 	}
@@ -269,7 +287,8 @@ TextWithEntities CashtagClickHandler::getExpandedLinkTextWithEntities(
 
 PeerData *BotCommandClickHandler::_peer = nullptr;
 UserData *BotCommandClickHandler::_bot = nullptr;
-void BotCommandClickHandler::onClick(Qt::MouseButton button) const {
+void BotCommandClickHandler::onClick(ClickContext context) const {
+	const auto button = context.button;
 	if (button == Qt::LeftButton || button == Qt::MiddleButton) {
 		if (auto peer = peerForCommand()) {
 			if (auto bot = peer->isUser() ? peer->asUser() : botForCommand()) {
