@@ -32,9 +32,6 @@ extern "C" {
 #endif // else of Q_OS_WIN
 
 namespace Core {
-
-#ifndef TDESKTOP_DISABLE_AUTOUPDATE
-
 namespace {
 
 constexpr auto kUpdaterTimeout = 10 * TimeMs(1000);
@@ -42,6 +39,12 @@ constexpr auto kMaxResponseSize = 1024 * 1024;
 constexpr auto kMaxUpdateSize = 256 * 1024 * 1024;
 constexpr auto kChunkSize = 128 * 1024;
 
+#ifdef TDESKTOP_DISABLE_AUTOUPDATE
+bool UpdaterIsDisabled = true;
+#else // TDESKTOP_DISABLE_AUTOUPDATE
+bool UpdaterIsDisabled = false;
+#endif // TDESKTOP_DISABLE_AUTOUPDATE
+
 std::weak_ptr<Updater> UpdaterInstance;
 
 using ErrorSignal = void(QNetworkReply::*)(QNetworkReply::NetworkError);
@@ -1254,7 +1257,7 @@ MtpChecker::MtpChecker(QPointer<MTP::Instance> instance, bool testing)
 void MtpChecker::start() {
 	if (!_mtp.valid()) {
 		LOG(("Update Info: MTP is unavailable."));
-		InvokeQueued(this, [=] { fail(); });
+		crl::on_main(this, [=] { fail(); });
 		return;
 	}
 	constexpr auto kFeedUsername = "tdhbcfeed";
@@ -1560,6 +1563,16 @@ Fn<void(const RPCError &)> MtpLoader::failHandler() {
 
 } // namespace
 
+bool UpdaterDisabled() {
+	return UpdaterIsDisabled;
+}
+
+void SetUpdaterDisabledAtStartup() {
+	Expects(UpdaterInstance.lock() == nullptr);
+
+	UpdaterIsDisabled = true;
+}
+
 class Updater : public base::has_weak_ptr {
 public:
 	Updater();
@@ -2099,35 +2112,32 @@ bool checkReadyUpdate() {
 }
 
 void UpdateApplication() {
-	cSetLastUpdateCheck(0);
-	Core::UpdateChecker().start();
-	if (const auto window = App::wnd()) {
-		auto settings = Box<Settings::Widget>();
-		const auto weak = make_weak(settings.data());
-		window->showSpecialLayer(std::move(settings), anim::type::normal);
-		if (weak) {
-			weak->scrollToUpdateRow();
+	if (UpdaterDisabled()) {
+		const auto url = [&] {
+#ifdef OS_WIN_STORE
+			return "https://www.microsoft.com/en-us/store/p/telegram-desktop/9nztwsqntd0s";
+#elif defined OS_MAC_STORE // OS_WIN_STORE
+			return "https://itunes.apple.com/ae/app/telegram-desktop/id946399090";
+#else // OS_WIN_STORE || OS_MAC_STORE
+			return "https://desktop.telegram.org";
+#endif // OS_WIN_STORE || OS_MAC_STORE
+		}();
+		UrlClickHandler::Open(url);
+	} else {
+		cSetLastUpdateCheck(0);
+		Core::UpdateChecker().start();
+		if (const auto window = App::wnd()) {
+			auto settings = Box<Settings::Widget>();
+			const auto weak = make_weak(settings.data());
+			window->showSpecialLayer(std::move(settings), anim::type::normal);
+			if (weak) {
+				weak->scrollToUpdateRow();
+			}
+			window->showFromTray();
 		}
 	}
 }
 
-#else // !TDESKTOP_DISABLE_AUTOUPDATE
-
-void UpdateApplication() {
-	const auto url = [&] {
-#ifdef OS_WIN_STORE
-		return "https://www.microsoft.com/en-us/store/p/telegram-desktop/9nztwsqntd0s";
-#elif defined OS_MAC_STORE // OS_WIN_STORE
-		return "https://itunes.apple.com/ae/app/telegram-desktop/id946399090";
-#else // OS_WIN_STORE || OS_MAC_STORE
-		return "https://desktop.telegram.org";
-#endif // OS_WIN_STORE || OS_MAC_STORE
-	}();
-	UrlClickHandler::doOpen(url);
-}
-
-#endif // !TDESKTOP_DISABLE_AUTOUPDATE
-
 QString countBetaVersionSignature(uint64 version) { // duplicated in packer.cpp
 	if (cBetaPrivateKey().isEmpty()) {
 		LOG(("Error: Trying to count beta version signature without beta private key!"));
