@@ -10,6 +10,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "observer_peer.h"
 #include "auth_session.h"
 #include "apiwrap.h"
+#include "export/export_controller.h"
+#include "export/view/export_view_panel_controller.h"
 #include "window/notifications_manager.h"
 #include "history/history.h"
 #include "history/history_item_components.h"
@@ -17,6 +19,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/view/history_view_element.h"
 #include "inline_bots/inline_bot_layout_item.h"
 #include "storage/localstorage.h"
+#include "boxes/abstract_box.h"
+#include "passport/passport_form_controller.h"
 #include "data/data_media_types.h"
 #include "data/data_feed.h"
 #include "data/data_photo.h"
@@ -68,6 +72,115 @@ Session::Session(not_null<AuthSession*> session)
 	setupChannelLeavingViewer();
 }
 
+void Session::startExport() {
+	if (_exportPanel) {
+		_exportPanel->activatePanel();
+		return;
+	}
+	_export = std::make_unique<Export::ControllerWrap>();
+	_exportPanel = std::make_unique<Export::View::PanelController>(
+		_export.get());
+
+	_exportViewChanges.fire(_exportPanel.get());
+
+	_exportPanel->stopRequests(
+	) | rpl::start_with_next([=] {
+		LOG(("Export Info: Stop requested."));
+		stopExport();
+	}, _export->lifetime());
+}
+
+void Session::suggestStartExport(TimeId availableAt) {
+	_exportAvailableAt = availableAt;
+	suggestStartExport();
+}
+
+void Session::clearExportSuggestion() {
+	_exportAvailableAt = 0;
+	if (_exportSuggestion) {
+		_exportSuggestion->closeBox();
+	}
+}
+
+void Session::suggestStartExport() {
+	if (_exportAvailableAt <= 0) {
+		return;
+	}
+
+	const auto now = unixtime();
+	const auto left = (_exportAvailableAt <= now)
+		? 0
+		: (_exportAvailableAt - now);
+	if (left) {
+		App::CallDelayed(
+			std::min(left + 5, 3600) * TimeMs(1000),
+			_session,
+			[=] { suggestStartExport(); });
+	} else if (_export) {
+		Export::View::ClearSuggestStart();
+	} else {
+		_exportSuggestion = Export::View::SuggestStart();
+	}
+}
+
+rpl::producer<Export::View::PanelController*> Session::currentExportView(
+) const {
+	return _exportViewChanges.events_starting_with(_exportPanel.get());
+}
+
+bool Session::exportInProgress() const {
+	return _export != nullptr;
+}
+
+void Session::stopExportWithConfirmation(FnMut<void()> callback) {
+	if (!_exportPanel) {
+		callback();
+		return;
+	}
+	auto closeAndCall = [=, callback = std::move(callback)]() mutable {
+		auto saved = std::move(callback);
+		LOG(("Export Info: Stop With Confirmation."));
+		stopExport();
+		if (saved) {
+			saved();
+		}
+	};
+	_exportPanel->stopWithConfirmation(std::move(closeAndCall));
+}
+
+void Session::stopExport() {
+	if (_exportPanel) {
+		LOG(("Export Info: Destroying."));
+		_exportPanel = nullptr;
+		_exportViewChanges.fire(nullptr);
+	}
+	_export = nullptr;
+}
+
+const Passport::SavedCredentials *Session::passportCredentials() const {
+	return _passportCredentials ? &_passportCredentials->first : nullptr;
+}
+
+void Session::rememberPassportCredentials(
+		Passport::SavedCredentials data,
+		TimeMs rememberFor) {
+	Expects(rememberFor > 0);
+
+	static auto generation = 0;
+	_passportCredentials = std::make_unique<CredentialsWithGeneration>(
+		std::move(data),
+		++generation);
+	App::CallDelayed(rememberFor, _session, [=, check = generation] {
+		if (_passportCredentials && _passportCredentials->second == check) {
+			forgetPassportCredentials();
+		}
+	});
+}
+
+void Session::forgetPassportCredentials() {
+	_passportCredentials = nullptr;
+}
+
 void Session::setupContactViewsViewer() {
 	Notify::PeerUpdateViewer(
 		Notify::PeerUpdate::Flag::UserIsContact
@@ -1002,7 +1115,7 @@ DocumentData *Session::documentFromWeb(
 		int32(0), // data.vsize.v
 		StorageImageLocation());
 	result->setWebLocation(WebFileLocation(
-		data.vdc_id.v,
+		Global::WebFileDcId(),
 		data.vurl.v,
 		data.vaccess_hash.v));
 	return result;
