@@ -13,6 +13,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "base/timer.h"
 
 class HistoryItem;
+class BoxContent;
 
 namespace HistoryView {
 struct Group;
@@ -27,6 +28,17 @@ class Reader;
 } // namespace Clip
 } // namespace Media
 
+namespace Export {
+class ControllerWrap;
+namespace View {
+class PanelController;
+} // namespace View
+} // namespace Export
+
+namespace Passport {
+struct SavedCredentials;
+} // namespace Passport
+
 namespace Data {
 
 class Feed;
@@ -44,6 +56,20 @@ public:
 		return *_session;
 	}
 
+	void startExport();
+	void suggestStartExport(TimeId availableAt);
+	void clearExportSuggestion();
+	rpl::producer<Export::View::PanelController*> currentExportView() const;
+	bool exportInProgress() const;
+	void stopExportWithConfirmation(FnMut<void()> callback);
+	void stopExport();
+
+	const Passport::SavedCredentials *passportCredentials() const;
+	void rememberPassportCredentials(
+		Passport::SavedCredentials data,
+		TimeMs rememberFor);
+	void forgetPassportCredentials();
+
 	[[nodiscard]] base::Variable<bool> &contactsLoaded() {
 		return _contactsLoaded;
 	}
@@ -395,6 +421,8 @@ public:
 	}
 
 private:
+	void suggestStartExport();
+
 	void setupContactViewsViewer();
 	void setupChannelLeavingViewer();
 	void photoApplyFields(
@@ -489,6 +517,12 @@ private:
 
 	not_null<AuthSession*> _session;
 
+	std::unique_ptr<Export::ControllerWrap> _export;
+	std::unique_ptr<Export::View::PanelController> _exportPanel;
+	rpl::event_stream<Export::View::PanelController*> _exportViewChanges;
+	TimeId _exportAvailableAt = 0;
+	QPointer<BoxContent> _exportSuggestion;
+
 	base::Variable<bool> _contactsLoaded = { false };
 	base::Variable<bool> _allChatsLoaded = { false };
 	base::Observable<void> _moreChatsLoaded;
@@ -586,6 +620,11 @@ private:
 
 	MessageIdsList _mimeForwardIds;
 
+	using CredentialsWithGeneration = std::pair<
+		const Passport::SavedCredentials,
+		int>;
+	std::unique_ptr<CredentialsWithGeneration> _passportCredentials;
+
 	rpl::lifetime _lifetime;
 
 };
