@@ -409,9 +409,9 @@ Data::Draft *History::createCloudDraft(Data::Draft *fromDraft) {
 }
 
 bool History::skipCloudDraft(const QString &text, TimeId date) const {
-	if (_lastSentDraftText && *_lastSentDraftText == text) {
+	if (date > 0 && date <= _lastSentDraftTime + kSkipCloudDraftsFor) {
 		return true;
-	} else if (date <= _lastSentDraftTime + kSkipCloudDraftsFor) {
+	} else if (_lastSentDraftText && *_lastSentDraftText == text) {
 		return true;
 	}
 	return false;
@@ -421,8 +421,10 @@ void History::setSentDraftText(const QString &text) {
 	_lastSentDraftText = text;
 }
 
-void History::clearSentDraftText() {
-	_lastSentDraftText = base::none;
+void History::clearSentDraftText(const QString &text) {
+	if (_lastSentDraftText && *_lastSentDraftText == text) {
+		_lastSentDraftText = base::none;
+	}
 	accumulate_max(_lastSentDraftTime, unixtime());
 }
 
@@ -1643,6 +1645,11 @@ int History::unreadCount() const {
 	return _unreadCount ? *_unreadCount : 0;
 }
 
+int History::historiesUnreadCount() const {
+	const auto result = unreadCount();
+	return (!result && unreadMark()) ? 1 : result;
+}
+
 bool History::unreadCountKnown() const {
 	return !!_unreadCount;
 }
@@ -1671,6 +1678,16 @@ void History::setUnreadCount(int newUnreadCount) {
 				calculateFirstUnreadMessage();
 			}
 		}
+		const auto unreadMarkDelta = [&] {
+			if (_unreadMark) {
+				const auto was = _unreadCount && (*_unreadCount > 0);
+				const auto now = (newUnreadCount > 0);
+				if (was != now) {
+					return was ? 1 : -1;
+				}
+			}
+			return 0;
+		}();
 		_unreadCount = newUnreadCount;
 
 		if (_unreadBarView) {
@@ -1683,16 +1700,40 @@ void History::setUnreadCount(int newUnreadCount) {
 		}
 
 		if (inChatList(Dialogs::Mode::All)) {
+			const auto delta = unreadCountDelta
+				? *unreadCountDelta
+				: newUnreadCount;
 			App::histories().unreadIncrement(
-				unreadCountDelta ? *unreadCountDelta : newUnreadCount,
+				delta + unreadMarkDelta,
 				mute());
 		}
-		if (const auto main = App::main()) {
-			main->unreadCountChanged(this);
-		}
+		Notify::peerUpdatedDelayed(
+			peer,
+			Notify::PeerUpdate::Flag::UnreadViewChanged);
 	}
 }
 
+void History::setUnreadMark(bool unread) {
+	if (_unreadMark != unread) {
+		_unreadMark = unread;
+		if (!_unreadCount || !*_unreadCount) {
+			if (inChatList(Dialogs::Mode::All)) {
+				const auto delta = _unreadMark ? 1 : -1;
+				App::histories().unreadIncrement(delta, mute());
+
+				updateChatListEntry();
+			}
+		}
+		Notify::peerUpdatedDelayed(
+			peer,
+			Notify::PeerUpdate::Flag::UnreadViewChanged);
+	}
+}
+
+bool History::unreadMark() const {
+	return _unreadMark;
+}
+
 void History::changeUnreadCount(int delta) {
 	if (_unreadCount) {
 		setUnreadCount(std::max(*_unreadCount + delta, 0));
@@ -1731,8 +1772,8 @@ bool History::changeMute(bool newMute) {
 		}
 	}
 	if (inChatList(Dialogs::Mode::All)) {
-		if (_unreadCount && *_unreadCount) {
-			App::histories().unreadMuteChanged(*_unreadCount, _mute);
+		if (const auto count = historiesUnreadCount()) {
+			App::histories().unreadMuteChanged(count, _mute);
 			Notify::unreadCounterUpdated();
 		}
 		Notify::historyMuteUpdated(this);
@@ -1945,8 +1986,7 @@ not_null<HistoryItem*> History::addNewInTheMiddle(
 	return item;
 }
 
-int History::chatListUnreadCount() const {
-	const auto result = unreadCount();
+History *History::migrateSibling() const {
 	const auto addFromId = [&] {
 		if (const auto from = peer->migrateFrom()) {
 			return from->id;
@@ -1955,12 +1995,26 @@ int History::chatListUnreadCount() const {
 		}
 		return PeerId(0);
 	}();
-	if (const auto migrated = App::historyLoaded(addFromId)) {
+	return App::historyLoaded(addFromId);
+}
+
+int History::chatListUnreadCount() const {
+	const auto result = unreadCount();
+	if (const auto migrated = migrateSibling()) {
 		return result + migrated->unreadCount();
 	}
 	return result;
 }
 
+bool History::chatListUnreadMark() const {
+	if (unreadMark()) {
+		return true;
+	} else if (const auto migrated = migrateSibling()) {
+		return migrated->unreadMark();
+	}
+	return false;
+}
+
 bool History::chatListMutedBadge() const {
 	return mute();
 }
@@ -2199,6 +2253,7 @@ void History::applyDialog(const MTPDdialog &data) {
 		data.vread_inbox_max_id.v,
 		data.vread_outbox_max_id.v);
 	applyDialogTopMessage(data.vtop_message.v);
+	setUnreadMark(data.is_unread_mark());
 	setUnreadMentionsCount(data.vunread_mentions_count.v);
 	if (const auto channel = peer->asChannel()) {
 		if (data.has_pts()) {
@@ -2648,9 +2703,10 @@ void History::applyGroupAdminChanges(
 }
 
 void History::changedInChatListHook(Dialogs::Mode list, bool added) {
-	if (list == Dialogs::Mode::All && unreadCount()) {
-		const auto delta = added ? unreadCount() : -unreadCount();
-		App::histories().unreadIncrement(delta, mute());
+	if (list == Dialogs::Mode::All) {
+		if (const auto delta = historiesUnreadCount() * (added ? 1 : -1)) {
+			App::histories().unreadIncrement(delta, mute());
+		}
 	}
 }
 
