@@ -24,6 +24,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/unread_badge.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/dropdown_menu.h"
+#include "ui/effects/radial_animation.h"
 #include "window/window_controller.h"
 #include "window/window_peer_menu.h"
 #include "calls/calls_instance.h"
@@ -124,10 +125,38 @@ TopBarWidget::TopBarWidget(
 		[this] { updateInfoToggleActive(); },
 		lifetime());
 
+	rpl::single(rpl::empty_value()) | rpl::then(
+		base::ObservableViewer(Global::RefConnectionTypeChanged())
+	) | rpl::start_with_next(
+		[=] { updateConnectingState(); },
+		lifetime());
+
 	setCursor(style::cur_pointer);
 	updateControlsVisibility();
 }
 
+void TopBarWidget::updateConnectingState() {
+	const auto mtp = MTP::dcstate();
+	if (mtp == MTP::ConnectedState) {
+		if (_connecting) {
+			_connecting = nullptr;
+			update();
+		}
+	} else if (!_connecting) {
+		_connecting = std::make_unique<Ui::InfiniteRadialAnimation>(
+			animation(this, &TopBarWidget::step_connecting),
+			st::topBarConnectingAnimation);
+		_connecting->start();
+		update();
+	}
+}
+
+void TopBarWidget::step_connecting(TimeMs ms, bool timer) {
+	if (timer) {
+		update();
+	}
+}
+
 void TopBarWidget::refreshLang() {
 	InvokeQueued(this, [this] { updateControlsGeometry(); });
 }
@@ -158,12 +187,12 @@ void TopBarWidget::showMenu() {
 			weak->_menuToggle->setForceRippled(false);
 		}
 	});
-	_menu->setShowStartCallback(base::lambda_guarded(this, [this, menu = _menu.data()] {
+	_menu->setShowStartCallback(crl::guard(this, [this, menu = _menu.data()] {
 		if (_menu == menu) {
 			_menuToggle->setForceRippled(true);
 		}
 	}));
-	_menu->setHideStartCallback(base::lambda_guarded(this, [this, menu = _menu.data()] {
+	_menu->setHideStartCallback(crl::guard(this, [this, menu = _menu.data()] {
 		if (_menu == menu) {
 			_menuToggle->setForceRippled(false);
 		}
@@ -171,7 +200,7 @@ void TopBarWidget::showMenu() {
 	_menuToggle->installEventFilter(_menu);
 	const auto addAction = [&](
 			const QString &text,
-			base::lambda<void()> callback) {
+			Fn<void()> callback) {
 		return _menu->addAction(text, std::move(callback));
 	};
 	if (const auto peer = _activeChat.peer()) {
@@ -300,24 +329,71 @@ void TopBarWidget::paintTopBar(Painter &p, TimeMs ms) {
 		history->peer->dialogName().drawElided(p, nameleft, nametop, namewidth);
 
 		p.setFont(st::dialogsTextFont);
-		if (!history->paintSendAction(p, nameleft, statustop, namewidth, width(), st::historyStatusFgTyping, ms)) {
-			auto statustext = _titlePeerText;
-			auto statuswidth = _titlePeerTextWidth;
-			if (statuswidth > namewidth) {
-				statustext = st::dialogsTextFont->elided(
-					statustext,
-					namewidth,
-					Qt::ElideLeft);
-				statuswidth = st::dialogsTextFont->width(statustext);
-			}
-			p.setPen(_titlePeerTextOnline
-				? st::historyStatusFgActive
-				: st::historyStatusFg);
-			p.drawTextLeft(nameleft, statustop, width(), statustext, statuswidth);
+		if (paintConnectingState(p, nameleft, statustop, width(), ms)) {
+			return;
+		} else if (history->paintSendAction(
+			p,
+			nameleft,
+			statustop,
+			namewidth,
+			width(),
+			st::historyStatusFgTyping,
+			ms)) {
+			return;
+		} else {
+			paintStatus(p, nameleft, statustop, namewidth, width());
 		}
 	}
 }
 
+bool TopBarWidget::paintConnectingState(
+		Painter &p,
+		int left,
+		int top,
+		int outerWidth,
+		TimeMs ms) {
+	if (_connecting) {
+		_connecting->step(ms);
+	}
+	if (!_connecting) {
+		return false;
+	}
+	_connecting->draw(
+		p,
+		{
+			st::topBarConnectingPosition.x() + left,
+			st::topBarConnectingPosition.y() + top
+		},
+		outerWidth);
+	left += st::topBarConnectingPosition.x()
+		+ st::topBarConnectingAnimation.size.width()
+		+ st::topBarConnectingSkip;
+	p.setPen(st::historyStatusFg);
+	p.drawTextLeft(left, top, outerWidth, lang(lng_status_connecting));
+	return true;
+}
+
+void TopBarWidget::paintStatus(
+		Painter &p,
+		int left,
+		int top,
+		int availableWidth,
+		int outerWidth) {
+	auto statustext = _titlePeerText;
+	auto statuswidth = _titlePeerTextWidth;
+	if (statuswidth > availableWidth) {
+		statustext = st::dialogsTextFont->elided(
+			statustext,
+			availableWidth,
+			Qt::ElideLeft);
+		statuswidth = st::dialogsTextFont->width(statustext);
+	}
+	p.setPen(_titlePeerTextOnline
+		? st::historyStatusFgActive
+		: st::historyStatusFg);
+	p.drawTextLeft(left, top, outerWidth, statustext, statuswidth);
+}
+
 QRect TopBarWidget::getMembersShowAreaGeometry() const {
 	int membersTextLeft = _leftTaken;
 	int membersTextTop = st::topBarHeight - st::topBarArrowPadding.bottom() - st::dialogsTextFont->height;
@@ -418,8 +494,9 @@ void TopBarWidget::updateControlsGeometry() {
 	buttonsWidth += buttonsLeft + st::topBarActionSkip * 3;
 
 	auto widthLeft = qMin(width() - buttonsWidth, -2 * st::defaultActiveButton.width);
-	_forward->setFullWidth(-(widthLeft / 2));
-	_delete->setFullWidth(-(widthLeft / 2));
+	auto buttonFullWidth = qMin(-(widthLeft / 2), 0);
+	_forward->setFullWidth(buttonFullWidth);
+	_delete->setFullWidth(buttonFullWidth);
 
 	selectedButtonsTop += (height() - _forward->height()) / 2;
 
@@ -765,4 +842,6 @@ void TopBarWidget::updateOnlineDisplayIn(TimeMs timeout) {
 	_onlineUpdater.callOnce(timeout);
 }
 
+TopBarWidget::~TopBarWidget() = default;
+
 } // namespace HistoryView
