@@ -63,9 +63,9 @@ private:
 	using ListWidget = PeerListContent;
 
 	// PeerListContentDelegate interface.
-	void peerListSetTitle(base::lambda<QString()> title) override;
+	void peerListSetTitle(Fn<QString()> title) override;
 	void peerListSetAdditionalTitle(
-		base::lambda<QString()> title) override;
+		Fn<QString()> title) override;
 	bool peerListIsRowSelected(not_null<PeerData*> peer) override;
 	int peerListSelectedRowsCount() override;
 	std::vector<not_null<PeerData*>> peerListCollectSelectedRows() override;
