@@ -29,7 +29,7 @@ public:
 // this type used as a flag, we dynamic_cast<> to it
 class SendClickHandler : public ClickHandler {
 public:
-	void onClick(Qt::MouseButton) const override {
+	void onClick(ClickContext context) const override {
 	}
 };
 
@@ -42,9 +42,13 @@ public:
 
 class ItemBase : public LayoutItemBase {
 public:
-	ItemBase(not_null<Context*> context, Result *result) : _result(result), _context(context) {
+	ItemBase(not_null<Context*> context, not_null<Result*> result)
+	: _result(result)
+	, _context(context) {
 	}
-	ItemBase(not_null<Context*> context, DocumentData *doc) : _doc(doc), _context(context) {
+	ItemBase(not_null<Context*> context, DocumentData *doc)
+	: _doc(doc)
+	, _context(context) {
 	}
 	// Not used anywhere currently.
 	//ItemBase(not_null<Context*> context, PhotoData *photo) : _photo(photo), _context(context) {
