@@ -466,9 +466,9 @@ void Messenger::startMtp() {
 	_langCloudManager = std::make_unique<Lang::CloudManager>(
 		langpack(),
 		mtp());
-#ifndef TDESKTOP_DISABLE_AUTOUPDATE
-	Core::UpdateChecker().setMtproto(mtp());
-#endif // TDESKTOP_DISABLE_AUTOUPDATE
+	if (!Core::UpdaterDisabled()) {
+		Core::UpdateChecker().setMtproto(mtp());
+	}
 }
 
 void Messenger::destroyMtpKeys(MTP::AuthKeysList &&keys) {
@@ -841,13 +841,13 @@ void Messenger::checkStartUrl() {
 	if (!cStartUrl().isEmpty() && !locked()) {
 		auto url = cStartUrl();
 		cSetStartUrl(QString());
-		if (!openLocalUrl(url)) {
+		if (!openLocalUrl(url, {})) {
 			cSetStartUrl(url);
 		}
 	}
 }
 
-bool Messenger::openLocalUrl(const QString &url) {
+bool Messenger::openLocalUrl(const QString &url, QVariant context) {
 	auto urlTrimmed = url.trimmed();
 	if (urlTrimmed.size() > 8192) urlTrimmed = urlTrimmed.mid(0, 8192);
 
@@ -936,7 +936,12 @@ bool Messenger::openLocalUrl(const QString &url) {
 					startToken = gameParam;
 					post = ShowAtGameShareMsgId;
 				}
-				main->openPeerByName(domain, post, startToken);
+				const auto clickFromMessageId = context.value<FullMsgId>();
+				main->openPeerByName(
+					domain,
+					post,
+					startToken,
+					clickFromMessageId);
 				return true;
 			}
 		}
