@@ -153,20 +153,6 @@ void Session::createDcData() {
 	connect(dc.get(), SIGNAL(connectionWasInited()), this, SLOT(connectionWasInitedForDC()), Qt::QueuedConnection);
 }
 
-void Session::registerRequest(mtpRequestId requestId, ShiftedDcId dcWithShift) {
-	return _instance->registerRequest(requestId, dcWithShift);
-}
-
-mtpRequestId Session::storeRequest(
-		mtpRequest &request,
-		RPCResponseHandler &&callbacks) {
-	return _instance->storeRequest(request, std::move(callbacks));
-}
-
-mtpRequest Session::getRequest(mtpRequestId requestId) {
-	return _instance->getRequest(requestId);
-}
-
 bool Session::rpcErrorOccured(mtpRequestId requestId, const RPCFailHandlerPtr &onFail, const RPCError &err) { // return true if need to clean request data
 	return _instance->rpcErrorOccured(requestId, onFail, err);
 }
@@ -280,9 +266,9 @@ void Session::needToResumeAndSend() {
 }
 
 void Session::sendPong(quint64 msgId, quint64 pingId) {
-	send(mtpRequestData::serialize(MTPPong(MTP_pong(
-		MTP_long(msgId),
-		MTP_long(pingId)))));
+	_instance->sendProtocolMessage(
+		dcWithShift,
+		MTPPong(MTP_pong(MTP_long(msgId), MTP_long(pingId))));
 }
 
 void Session::sendMsgsStateInfo(quint64 msgId, QByteArray data) {
@@ -291,8 +277,10 @@ void Session::sendMsgsStateInfo(quint64 msgId, QByteArray data) {
 		info.resize(data.size());
 		bytes::copy(info, bytes::make_span(data));
 	}
-	send(mtpRequestData::serialize(MTPMsgsStateInfo(
-		MTP_msgs_state_info(MTP_long(msgId), MTP_bytes(data)))));
+	_instance->sendProtocolMessage(
+		dcWithShift,
+		MTPMsgsStateInfo(
+			MTP_msgs_state_info(MTP_long(msgId), MTP_bytes(data))));
 }
 
 void Session::checkRequestsByTimer() {
@@ -302,14 +290,14 @@ void Session::checkRequestsByTimer() {
 
 	{
 		QReadLocker locker(data.haveSentMutex());
-		mtpRequestMap &haveSent(data.haveSentMap());
-		uint32 haveSentCount(haveSent.size());
+		auto &haveSent = data.haveSentMap();
+		const auto haveSentCount = haveSent.size();
 		auto ms = getms(true);
-		for (mtpRequestMap::iterator i = haveSent.begin(), e = haveSent.end(); i != e; ++i) {
-			mtpRequest &req(i.value());
+		for (auto i = haveSent.begin(), e = haveSent.end(); i != e; ++i) {
+			auto &req = i.value();
 			if (req->msDate > 0) {
 				if (req->msDate + MTPCheckResendTimeout < ms) { // need to resend or check state
-					if (mtpRequestData::messageSize(req) < MTPResendThreshold) { // resend
+					if (req.messageSize() < MTPResendThreshold) { // resend
 						resendingIds.reserve(haveSentCount);
 						resendingIds.push_back(i.key());
 					} else {
@@ -408,8 +396,8 @@ int32 Session::requestState(mtpRequestId requestId) const {
 	if (!requestId) return MTP::RequestSent;
 
 	QWriteLocker locker(data.toSendMutex());
-	const mtpPreRequestMap &toSend(data.toSendMap());
-	mtpPreRequestMap::const_iterator i = toSend.constFind(requestId);
+	const auto &toSend = data.toSendMap();
+	const auto i = toSend.constFind(requestId);
 	if (i != toSend.cend()) {
 		return MTP::RequestSending;
 	} else {
@@ -445,10 +433,10 @@ QString Session::transport() const {
 }
 
 mtpRequestId Session::resend(quint64 msgId, qint64 msCanWait, bool forceContainer, bool sendMsgStateInfo) {
-	mtpRequest request;
+	SecureRequest request;
 	{
 		QWriteLocker locker(data.haveSentMutex());
-		mtpRequestMap &haveSent(data.haveSentMap());
+		auto &haveSent = data.haveSentMap();
 
 		auto i = haveSent.find(msgId);
 		if (i == haveSent.end()) {
@@ -457,10 +445,12 @@ mtpRequestId Session::resend(quint64 msgId, qint64 msCanWait, bool forceContaine
 				DEBUG_LOG(("Message Info: cant resend %1, request not found").arg(msgId));
 
 				auto info = std::string(cantResend, cantResend + 1);
-				return send(mtpRequestData::serialize(MTPMsgsStateInfo(
-					MTP_msgs_state_info(
-						MTP_long(msgId),
-						MTP_string(std::move(info))))));
+				return _instance->sendProtocolMessage(
+					dcWithShift,
+					MTPMsgsStateInfo(
+						MTP_msgs_state_info(
+							MTP_long(msgId),
+							MTP_string(std::move(info)))));
 			}
 			return 0;
 		}
@@ -468,14 +458,14 @@ mtpRequestId Session::resend(quint64 msgId, qint64 msCanWait, bool forceContaine
 		request = i.value();
 		haveSent.erase(i);
 	}
-	if (mtpRequestData::isSentContainer(request)) { // for container just resend all messages we can
+	if (request.isSentContainer()) { // for container just resend all messages we can
 		DEBUG_LOG(("Message Info: resending container from haveSent, msgId %1").arg(msgId));
 		const mtpMsgId *ids = (const mtpMsgId *)(request->constData() + 8);
 		for (uint32 i = 0, l = (request->size() - 8) >> 1; i < l; ++i) {
 			resend(ids[i], 10, true);
 		}
 		return 0xFFFFFFFF;
-	} else if (!mtpRequestData::isStateRequest(request)) {
+	} else if (!request.isStateRequest()) {
 		request->msDate = forceContainer ? 0 : getms(true);
 		sendPrepared(request, msCanWait, false);
 		{
@@ -498,10 +488,12 @@ void Session::resendAll() {
 	QVector<mtpMsgId> toResend;
 	{
 		QReadLocker locker(data.haveSentMutex());
-		const mtpRequestMap &haveSent(data.haveSentMap());
+		const auto &haveSent = data.haveSentMap();
 		toResend.reserve(haveSent.size());
-		for (mtpRequestMap::const_iterator i = haveSent.cbegin(), e = haveSent.cend(); i != e; ++i) {
-			if (i.value()->requestId) toResend.push_back(i.key());
+		for (auto i = haveSent.cbegin(), e = haveSent.cend(); i != e; ++i) {
+			if (i.value()->requestId) {
+				toResend.push_back(i.key());
+			}
 		}
 	}
 	for (uint32 i = 0, l = toResend.size(); i < l; ++i) {
@@ -509,31 +501,12 @@ void Session::resendAll() {
 	}
 }
 
-mtpRequestId Session::send(
-		mtpRequest &&request,
-		RPCResponseHandler &&callbacks,
+void Session::sendPrepared(
+		const SecureRequest &request,
 		TimeMs msCanWait,
-		bool needsLayer,
-		bool toMainDC,
-		mtpRequestId after) {
-	DEBUG_LOG(("MTP Info: adding request to toSendMap, msCanWait %1").arg(msCanWait));
-
-	request->msDate = getms(true); // > 0 - can send without container
-	request->needsLayer = needsLayer;
-	if (after) {
-		request->after = getRequest(after);
-	}
-	const auto requestId = storeRequest(request, std::move(callbacks));
-	Assert(requestId != 0);
-
-	const auto signedDcId = toMainDC ? -getDcWithShift() : getDcWithShift();
-	sendPrepared(request, msCanWait);
-	registerRequest(requestId, signedDcId);
-
-	return requestId;
-}
-
-void Session::sendPrepared(const mtpRequest &request, TimeMs msCanWait, bool newRequest) { // returns true, if emit of needToSend() is needed
+		bool newRequest) {
+	DEBUG_LOG(("MTP Info: adding request to toSendMap, msCanWait %1"
+		).arg(msCanWait));
 	{
 		QWriteLocker locker(data.toSendMutex());
 		data.toSendMap().insert(request->requestId, request);
@@ -625,7 +598,7 @@ void Session::tryToReceive() {
 			}
 		}
 		if (isUpdate) {
-			if (dcWithShift == bareDcId(dcWithShift)) { // call globalCallback only in main session
+			if (dcWithShift == BareDcId(dcWithShift)) { // call globalCallback only in main session
 				_instance->globalCallback(message.constData(), message.constData() + message.size());
 			}
 		} else {
