@@ -117,7 +117,7 @@ struct EditScans::SpecialScan {
 	QPointer<Info::Profile::Button> upload;
 	bool errorShown = false;
 	Animation errorAnimation;
-
+	rpl::variable<bool> rowCreated;
 };
 
 ScanButton::ScanButton(
@@ -390,10 +390,14 @@ void EditScans::setupScans(const QString &header) {
 }
 
 void EditScans::setupSpecialScans(std::map<SpecialFile, ScanInfo> &&files) {
-	const auto title = [](SpecialFile type) {
+	const auto requiresBothSides = files.find(SpecialFile::ReverseSide)
+		!= end(files);
+	const auto title = [&](SpecialFile type) {
 		switch (type) {
 		case SpecialFile::FrontSide:
-			return lang(lng_passport_front_side_title);
+			return lang(requiresBothSides
+				? lng_passport_front_side_title
+				: lng_passport_main_page_title);
 		case SpecialFile::ReverseSide:
 			return lang(lng_passport_reverse_side_title);
 		case SpecialFile::Selfie:
@@ -401,21 +405,33 @@ void EditScans::setupSpecialScans(std::map<SpecialFile, ScanInfo> &&files) {
 		}
 		Unexpected("Type in special row title.");
 	};
-	const auto uploadKey = [](SpecialFile type) {
+	const auto uploadKey = [=](SpecialFile type, bool hasScan) {
 		switch (type) {
 		case SpecialFile::FrontSide:
-			return lng_passport_upload_front_side;
+			return requiresBothSides
+				? (hasScan
+					? lng_passport_reupload_front_side
+					: lng_passport_upload_front_side)
+				: (hasScan
+					? lng_passport_reupload_main_page
+					: lng_passport_upload_main_page);
 		case SpecialFile::ReverseSide:
-			return lng_passport_upload_reverse_side;
+			return hasScan
+				? lng_passport_reupload_reverse_side
+				: lng_passport_upload_reverse_side;
 		case SpecialFile::Selfie:
-			return lng_passport_upload_selfie;
+			return hasScan
+				? lng_passport_reupload_selfie
+				: lng_passport_upload_selfie;
 		}
 		Unexpected("Type in special row upload key.");
 	};
-	const auto description = [](SpecialFile type) {
+	const auto description = [&](SpecialFile type) {
 		switch (type) {
 		case SpecialFile::FrontSide:
-			return lang(lng_passport_front_side_description);
+			return lang(requiresBothSides
+				? lng_passport_front_side_description
+				: lng_passport_main_page_description);
 		case SpecialFile::ReverseSide:
 			return lang(lng_passport_reverse_side_description);
 		case SpecialFile::Selfie:
@@ -444,14 +460,17 @@ void EditScans::setupSpecialScans(std::map<SpecialFile, ScanInfo> &&files) {
 		scan.header->toggle(scan.file.key.id != 0, anim::type::instant);
 		scan.wrap = inner->add(object_ptr<Ui::VerticalLayout>(inner));
 		if (scan.file.key.id) {
-			createSpecialScanRow(scan, scan.file);
+			createSpecialScanRow(scan, scan.file, requiresBothSides);
 		}
+		auto label = scan.rowCreated.value(
+		) | rpl::map([=, type = type](bool created) {
+			return Lang::Viewer(uploadKey(type, created));
+		}) | rpl::flatten_latest(
+		) | Info::Profile::ToUpperValue();
 		scan.upload = inner->add(
 			object_ptr<Info::Profile::Button>(
 				inner,
-				Lang::Viewer(
-					uploadKey(type)
-				) | Info::Profile::ToUpperValue(),
+				std::move(label),
 				st::passportUploadButton),
 			st::passportUploadButtonPadding);
 		scan.upload->addClickHandler([=, type = type] {
@@ -527,11 +546,15 @@ void EditScans::updateSpecialScan(SpecialFile type, ScanInfo &&info) {
 	auto &scan = i->second;
 	if (scan.file.key.id) {
 		updateFileRow(scan.row->entity(), info);
+		scan.rowCreated = !info.deleted;
 		if (!info.deleted) {
 			hideSpecialScanError(type);
 		}
 	} else {
-		createSpecialScanRow(scan, info);
+		const auto requiresBothSides
+			= (_specialScans.find(SpecialFile::ReverseSide)
+				!= end(_specialScans));
+		createSpecialScanRow(scan, info, requiresBothSides);
 		scan.wrap->resizeToWidth(width());
 		scan.row->show(anim::type::normal);
 		scan.header->show(anim::type::normal);
@@ -551,14 +574,17 @@ void EditScans::updateFileRow(
 
 void EditScans::createSpecialScanRow(
 		SpecialScan &scan,
-		const ScanInfo &info) {
+		const ScanInfo &info,
+		bool requiresBothSides) {
 	Expects(scan.file.special.has_value());
 
 	const auto type = *scan.file.special;
 	const auto name = [&] {
 		switch (type) {
 		case SpecialFile::FrontSide:
-			return lang(lng_passport_front_side_name);
+			return lang(requiresBothSides
+				? lng_passport_front_side_name
+				: lng_passport_main_page_name);
 		case SpecialFile::ReverseSide:
 			return lang(lng_passport_reverse_side_name);
 		case SpecialFile::Selfie:
@@ -579,6 +605,7 @@ void EditScans::createSpecialScanRow(
 		_controller->restoreSpecialScan(type);
 	}, row->lifetime());
 
+	scan.rowCreated = !info.deleted;
 	hideSpecialScanError(type);
 }
 
@@ -632,7 +659,7 @@ void EditScans::chooseScan() {
 		_controller->uploadScan(std::move(content));
 	}, [=](ReadScanError error) {
 		_controller->readScanError(error);
-	});
+	}, true);
 }
 
 void EditScans::chooseSpecialScan(SpecialFile type) {
@@ -640,24 +667,33 @@ void EditScans::chooseSpecialScan(SpecialFile type) {
 		_controller->uploadSpecialScan(type, std::move(content));
 	}, [=](ReadScanError error) {
 		_controller->readScanError(error);
-	});
+	}, false);
 }
 
 void EditScans::ChooseScan(
 		QPointer<QWidget> parent,
 		Fn<void(QByteArray&&)> doneCallback,
-		Fn<void(ReadScanError)> errorCallback) {
+		Fn<void(ReadScanError)> errorCallback,
+		bool allowMany) {
 	Expects(parent != nullptr);
 
+	const auto processFiles = std::make_shared<Fn<void(QStringList&&)>>();
 	const auto filter = FileDialog::AllFilesFilter()
 		+ qsl(";;Image files (*")
 		+ cImgExtensions().join(qsl(" *"))
 		+ qsl(")");
 	const auto guardedCallback = crl::guard(parent, doneCallback);
 	const auto guardedError = crl::guard(parent, errorCallback);
-	const auto onMainCallback = [=](QByteArray content) {
-		crl::on_main([=, bytes = std::move(content)]() mutable {
+	const auto onMainCallback = [=](
+			QByteArray &&content,
+			QStringList &&remainingFiles) {
+		crl::on_main([
+			=,
+			bytes = std::move(content),
+			remainingFiles = std::move(remainingFiles)
+		]() mutable {
 			guardedCallback(std::move(bytes));
+			(*processFiles)(std::move(remainingFiles));
 		});
 	};
 	const auto onMainError = [=](ReadScanError error) {
@@ -665,22 +701,38 @@ void EditScans::ChooseScan(
 			guardedError(error);
 		});
 	};
-	const auto processImage = [=](QByteArray &&content) {
-		crl::async([=, bytes = std::move(content)]() mutable {
+	const auto processImage = [=](
+			QByteArray &&content,
+			QStringList &&remainingFiles) {
+		crl::async([
+			=,
+			bytes = std::move(content),
+			remainingFiles = std::move(remainingFiles)
+		]() mutable {
 			auto result = ProcessImage(std::move(bytes));
 			if (const auto error = base::get_if<ReadScanError>(&result)) {
 				onMainError(*error);
 			} else {
 				auto content = base::get_if<QByteArray>(&result);
 				Assert(content != nullptr);
-				onMainCallback(std::move(*content));
+				onMainCallback(std::move(*content), std::move(remainingFiles));
 			}
 		});
 	};
-	const auto processFile = [=](FileDialog::OpenResult &&result) {
-		if (result.paths.size() == 1) {
+	const auto processOpened = [=](FileDialog::OpenResult &&result) {
+		if (result.paths.size() > 0) {
+			(*processFiles)(std::move(result.paths));
+		} else if (!result.remoteContent.isEmpty()) {
+			processImage(std::move(result.remoteContent), {});
+		}
+	};
+	*processFiles = [=](QStringList &&files) {
+		while (!files.isEmpty()) {
+			auto file = files.front();
+			files.removeAt(0);
+
 			auto content = [&] {
-				QFile f(result.paths.front());
+				QFile f(file);
 				if (f.size() > App::kImageSizeLimit) {
 					guardedError(ReadScanError::FileTooLarge);
 					return QByteArray();
@@ -691,17 +743,17 @@ void EditScans::ChooseScan(
 				return f.readAll();
 			}();
 			if (!content.isEmpty()) {
-				processImage(std::move(content));
+				processImage(std::move(content), std::move(files));
+				return;
 			}
-		} else if (!result.remoteContent.isEmpty()) {
-			processImage(std::move(result.remoteContent));
 		}
 	};
-	FileDialog::GetOpenPath(
+	(allowMany ? FileDialog::GetOpenPaths : FileDialog::GetOpenPath)(
 		parent,
 		lang(lng_passport_choose_image),
 		filter,
-		processFile);
+		processOpened,
+		nullptr);
 }
 
 rpl::producer<QString> EditScans::uploadButtonText() const {
