@@ -8,6 +8,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #pragma once
 
 #include "base/optional.h"
+#include "base/variant.h"
 #include <rpl/map.h>
 #include <rpl/producer.h>
 #include <rpl/details/type_list.h>
@@ -121,7 +122,7 @@ template <
 class combine_implementation_helper<producer<Values, Errors, Generators>...> {
 public:
 	using CombinedValue = std::tuple<Values...>;
-	using CombinedError = normalized_variant_t<Errors...>;
+	using CombinedError = base::normalized_variant_t<Errors...>;
 
 	combine_implementation_helper(
 		producer<Values, Errors, Generators> &&...producers)
@@ -154,7 +155,7 @@ template <
 inline auto combine_implementation(
 		producer<Values, Errors, Generators> &&...producers) {
 	using CombinedValue = std::tuple<Values...>;
-	using CombinedError = normalized_variant_t<Errors...>;
+	using CombinedError = base::normalized_variant_t<Errors...>;
 
 	return make_producer<CombinedValue, CombinedError>(
 		make_combine_implementation_helper(std::move(producers)...));
