@@ -226,8 +226,8 @@ private:
 	void requestMoreCdnFileHashes();
 	void getCdnFileHashesDone(const MTPVector<MTPFileHash> &result, mtpRequestId requestId);
 
-	bool feedPart(int offset, base::const_byte_span bytes);
-	void partLoaded(int offset, base::const_byte_span bytes);
+	bool feedPart(int offset, bytes::const_span buffer);
+	void partLoaded(int offset, bytes::const_span buffer);
 
 	bool partFailed(const RPCError &error);
 	bool cdnPartFailed(const RPCError &error, mtpRequestId requestId);
@@ -243,7 +243,7 @@ private:
 		Invalid,
 		Good,
 	};
-	CheckCdnHashResult checkCdnFileHash(int offset, base::const_byte_span bytes);
+	CheckCdnHashResult checkCdnFileHash(int offset, bytes::const_span buffer);
 
 	std::map<mtpRequestId, RequestData> _sentRequests;
 
@@ -325,10 +325,6 @@ class WebLoadManager : public QObject {
 public:
 	WebLoadManager(QThread *thread);
 
-#ifndef TDESKTOP_DISABLE_NETWORK_PROXY
-	void setProxySettings(const QNetworkProxy &proxy);
-#endif // !TDESKTOP_DISABLE_NETWORK_PROXY
-
 	void append(webFileLoader *loader, const QString &url);
 	void stop(webFileLoader *reader);
 	bool carries(webFileLoader *reader) const;
@@ -337,7 +333,6 @@ public:
 
 signals:
 	void processDelayed();
-	void proxyApplyDelayed();
 
 	void progress(webFileLoader *loader, qint64 already, qint64 size);
 	void finished(webFileLoader *loader, QByteArray data);
@@ -350,7 +345,6 @@ public slots:
 	void onMeta();
 
 	void process();
-	void proxyApply();
 	void finish();
 
 private:
@@ -358,9 +352,6 @@ private:
 	void sendRequest(webFileLoaderPrivate *loader, const QString &redirect = QString());
 	bool handleReplyResult(webFileLoaderPrivate *loader, WebReplyProcessResult result);
 
-#ifndef TDESKTOP_DISABLE_NETWORK_PROXY
-	QNetworkProxy _proxySettings;
-#endif // !TDESKTOP_DISABLE_NETWORK_PROXY
 	QNetworkAccessManager _manager;
 	typedef QMap<webFileLoader*, webFileLoaderPrivate*> LoaderPointers;
 	LoaderPointers _loaderPointers;
@@ -392,5 +383,4 @@ static mtpFileLoader * const CancelledMtpFileLoader = static_cast<mtpFileLoader*
 static webFileLoader * const CancelledWebFileLoader = static_cast<webFileLoader*>(CancelledFileLoader);
 static WebLoadManager * const FinishedWebLoadManager = SharedMemoryLocation<WebLoadManager, 0>();
 
-void reinitWebLoadManager();
 void stopWebLoadManager();
